#include "amt/raycaster.hpp"
#include <iostream>
#include <chrono>
#include <numeric>
#include <functional>
#include "constants.hpp"
#include "stats.hpp"

Matrix MAP{
  {1,1,1,1,1,1,1,1,1},
  {1,0,2,0,0,0,0,0,1},
  {1,0,4,0,0,5,2,0,1},
  {1,0,0,0,0,0,0,0,1},
  {1,1,0,0,0,0,0,1,1},
  {1,0,0,1,3,4,0,0,1},
  {1,0,0,0,0,0,1,1,1},
  {1,0,0,0,0,0,0,0,1},
  {1,1,1,1,1,1,1,1,1}
};

void draw_gui(sf::RenderWindow &window, sf::Text &text, Stats &stats) {
  sf::RectangleShape rect({SCREEN_WIDTH - RAY_VIEW_WIDTH, SCREEN_HEIGHT});

  rect.setPosition({0,0});
  rect.setFillColor({50, 50, 50});
  window.draw(rect);

  text.setString(
    fmt::format("FPS\nmean:{:>8.5}\nsdev: {:>8.5}\nmin: {:>8.5}\nmax: {:>8.5}\ncount:{:<10}\n\nVSync? {}\nDebug? {}\n\nHit R to reset.",
      stats.mean(), stats.stddev(), stats.min, stats.max, stats.n, VSYNC, DEBUG_BUILD));
  window.draw(text);
}

int main() {
  sf::RenderWindow window(sf::VideoMode({SCREEN_WIDTH, SCREEN_HEIGHT}), "Zed's Ray Caster Game Thing");

  sf::Font font{"./assets/text.otf"};
  sf::Text text{font};
  text.setFillColor({255,255,255});
  text.setPosition({10,10});

  //ZED this should set with a function
  float player_x = MAP.rows() / 2;
  float player_y = MAP.cols() / 2;

  Raycaster rayview(window, MAP, RAY_VIEW_WIDTH, RAY_VIEW_HEIGHT);
  rayview.set_position(RAY_VIEW_X, RAY_VIEW_Y);
  rayview.position_camera(player_x, player_y);

  double moveSpeed = 0.1;
  double rotSpeed = 0.1;

  const auto onClose = [&window](const sf::Event::Closed&)
  {
      window.close();
  };

  Stats stats;

  // NOTE: Enable this to lock frames at 60
  window.setFramerateLimit(60);

  while(window.isOpen()) {
    auto start = std::chrono::high_resolution_clock::now();
    rayview.render();
    auto end = std::chrono::high_resolution_clock::now();
    auto elapsed = std::chrono::duration<double>(end - start);
    stats.sample(1/elapsed.count());

    draw_gui(window, text, stats);
    window.display();

    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::W)) {
      rayview.run(moveSpeed, 1);
    } else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::S)) {
      rayview.run(moveSpeed, -1);
    }

    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::D)) {
      rayview.rotate(rotSpeed, -1);
    } else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::A)) {
      rayview.rotate(rotSpeed, 1);
    }

    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::R)) {
      stats.reset();
    }

    window.handleEvents(onClose);
  }

  return 0;
}
