#include <catch2/catch_test_macros.hpp>
#include <iostream>
#include "rituals.hpp"
#include "battle.hpp"
#include "simplefsm.hpp"
#include "dinkyecs.hpp"

using namespace combat;

TEST_CASE("battle operations fantasy", "[combat-battle]") {
  ai::reset();
  ai::init("assets/ai.json");

  auto ai_start = ai::load_state("Enemy::initial_state");
  auto ai_goal = ai::load_state("Enemy::final_state");
  auto host_start = ai::load_state("Host::initial_state");
  auto host_goal = ai::load_state("Host::final_state");
  BattleEngine battle;

  DinkyECS::Entity player = 0;
  ai::EntityAI player_ai("Host::actions", host_start, host_goal);
  components::Combat player_combat{100, 100, 20};
  battle.add_enemy({player, player_ai, player_combat});

  DinkyECS::Entity axe_ranger = 1;
  ai::EntityAI axe_ai("Enemy::actions", ai_start, ai_goal);
  components::Combat axe_combat{100, 100, 20};
  battle.add_enemy({axe_ranger, axe_ai, axe_combat});

  DinkyECS::Entity rat = 2;
  ai::EntityAI rat_ai("Enemy::actions", ai_start, ai_goal);
  components::Combat rat_combat{10, 10, 2};
  battle.add_enemy({rat, rat_ai, rat_combat});

  battle.set_all("enemy_found", true);
  battle.set_all("in_combat", true);
  battle.set_all("tough_personality", true);
  battle.set_all("health_good", true);
  battle.set(rat, "tough_personality", false);

  battle.queue(player, BattleAction::ATTACK);
  battle.queue(player, BattleAction::BLOCK);

  battle.queue(player, BattleAction::ESCAPE);

  battle.plan();

  while(auto act = battle.next()) {
    auto& [enemy, action] = *act;

    fmt::println("entity: {} wants to {} action={} and has {} HP and {} damage",
        enemy.entity, enemy.ai.wants_to(),
        int(action), enemy.combat.hp,
        enemy.combat.damage);
  }

  REQUIRE(!battle.next());
}
